@@ -20,13 +20,15 @@ come with the distribution.
 
 =cut
 
-$VERSION = '0.96_02';
+$VERSION = '0.99_01';
 use strict;
 use B;
 use Sepia::Debug;               # THIS TURNS ON DEBUGGING INFORMATION!
 use Cwd 'abs_path';
 use Scalar::Util 'looks_like_number';
 use Text::Abbrev;
+use File::Find;
+use Storable qw(store retrieve);
 
 use vars qw($PS1 %REPL %RK %REPL_DOC %REPL_SHORT %PRINTER
             @REPL_RESULT @res
@@ -82,12 +84,13 @@ sub repl_size
         print "Size requires Devel::Size.\n";
     } else {
         *Sepia::repl_size = sub {
+            no strict 'refs';
             ## XXX: C&P from repl_who:
             my ($pkg, $re) = split ' ', shift || '';
             if ($pkg =~ /^\/(.*)\/?$/) {
                 $pkg = $PACKAGE;
                 $re = $1;
-            } elsif (!$re && !defined %{$pkg.'::'}) {
+            } elsif (!$re && !%{$pkg.'::'}) {
                 $re = $pkg;
                 $pkg = $PACKAGE;
             } else {
@@ -95,17 +98,18 @@ sub repl_size
                 $pkg = $PACKAGE;
             }
             my @who = who($pkg, $re);
-            my $len = max(map { length } @who) + 4;
+            my $len = max(3, map { length } @who) + 4;
             my $fmt = '%-'.$len."s%10d\n";
             print 'Var', ' ' x ($len + 2), "Bytes\n";
             print '-' x ($len-4), ' ' x 9, '-' x 5, "\n";
-            local $SIG{__WARN__} = sub {};
+            my %res;
             for (@who) {
                 next unless /^[\$\@\%\&]/; # skip subs.
-                # print STDERR "package $pkg; Devel::Size::total_size \\$_;";
-                my $res = eval "package $pkg; Devel::Size::total_size \\$_;";
-                # next if $res == 0;
-                printf $fmt, $_, $res;
+                next if $_ eq '%SIG';
+                $res{$_} = eval "no strict; package $pkg; Devel::Size::total_size \\$_;";
+            }
+            for (sort { $res{$b} <=> $res{$a} } keys %res) {
+                printf $fmt, $_, $res{$_};
             }
         };
         goto &repl_size;
@@ -119,7 +123,7 @@ development.  This package contains the Perl side of the
 implementation, including all user-serviceable parts (for the
 cross-referencing facility see L<Sepia::Xref>).  This document is
 aimed as Sepia developers; for user documentation, see
-L<sepia/index.html>.
+L<Sepia.html> or L<sepia.info>.
 
 Though not intended to be used independent of the Emacs interface, the
 Sepia module's functionality can be used through a rough procedural
@@ -171,7 +175,7 @@ sub filter_untyped
 {
     no strict;
     local $_ = /^::/ ? $_ : "::$_";
-    defined *{$_}{CODE} || defined *{$_}{IO} || (/::$/ && defined *{$_}{HASH});
+    defined *{$_}{CODE} || defined *{$_}{IO} || (/::$/ && %$_);
 }
 
 ## XXX: Careful about autovivification here!  Specifically:
@@ -184,9 +188,9 @@ sub filter_typed
     my $type = shift;
     local $_ = /^::/ ? $_ : "::$_";
     if ($type eq 'SCALAR') {
-        defined ${$_};
+        defined $$_;
     } elsif ($type eq 'VARIABLE') {
-        defined ${$_} || defined *{$_}{HASH} || defined *{$_}{ARRAY};
+        defined $$_ || defined *{$_}{HASH} || defined *{$_}{ARRAY};
     } else {
         defined *{$_}{$type}
     }
@@ -195,6 +199,7 @@ sub filter_typed
 sub maybe_icase
 {
     my $ch = shift;
+    return '' if $ch eq '';
     $ch =~ /[A-Z]/ ? $ch : '['.uc($ch).$ch.']';
 }
 
@@ -247,24 +252,45 @@ sub all_completions
     map { s/^:://; $_ } _completions('::', @parts);
 }
 
+sub lexical_completions
+{
+    eval { require PadWalker; import PadWalker 'peek_sub' };
+    # "internal" function, so don't warn on failure
+    return if $@;
+    *lexical_completions = sub {
+        my ($type, $str, $sub) = @_;
+        $sub = "$PACKAGE\::$sub" unless $sub =~ /::/;
+        # warn "Completing $str of type $type in $sub\n";
+        no strict;
+        return unless defined *{$sub}{CODE};
+        my $pad = peek_sub(\&$sub);
+        if ($type) {
+            map { s/^[\$\@&\%]//;$_ } grep /^\Q$type$str\E/, keys %$pad;
+        } else {
+            map { s/^[\$\@&\%]//;$_ } grep /^.\Q$str\E/, keys %$pad;
+        }
+    };
+    goto &lexical_completions;
+}
+
 sub completions
 {
-    my ($type, $str, $t);
+    my ($type, $str, $sub) = @_;
+    my $t;
     my %h = qw(@ ARRAY % HASH & CODE * IO $ SCALAR);
     my %rh;
     @rh{values %h} = keys %h;
-    if (@_ == 1) {
-        ($type, $str) = $_[0] =~ /^([\%\$\@\&]?)(.*)/;
-        $t = $type || '';
-    $type = $h{$type} if $type;
-    } else {
-        ($str, $type) = @_;
-        $type ||= '';
-        $t = $rh{$type} if $type;
+    $type ||= '';
+    $t = $type ? $rh{$type} : '';
+    my @ret;
+    if ($sub && $type ne '') {
+        @ret = lexical_completions $t, $str, $sub;
+    }
+    if (!@ret) {
+        @ret = grep {
+            $type ? filter_typed $type : filter_untyped
+        } all_completions $str;
     }
-    my @ret = grep {
-        $type ? filter_typed $type : filter_untyped
-    } all_completions $str;
     if (!@ret && $str !~ /:/) {
         @ret = grep {
             $type ? filter_typed $type : filter_untyped
@@ -274,7 +300,7 @@ sub completions
 #     ## XXX: Control characters, $", and $1, etc. confuse Emacs, so
 #     ## remove them.
     grep {
-        length > 0 && !looks_like_number $_ && !/^[^\w\d_]$/ && !/^_</ && !/^[[:cntrl:]]/
+        length $_ > 0 && !looks_like_number($_) && !/^[^\w\d_]$/ && !/^_</ && !/^[[:cntrl:]]/
     } @ret;
 }
 
@@ -286,7 +312,7 @@ sub method_completions
     $eval ||= 'CORE::eval';
     no strict;
     return unless ($x =~ /^\$/ && ($x = $eval->("ref($x)")))
-        || $eval->('defined(%{'.$x.'::})');
+        || $eval->('%'.$x.'::');
     unless ($@) {
         my $re = _apropos_re $fn;
         ## Filter out overload methods "(..."
@@ -375,7 +401,7 @@ sub apropos
     no strict;
     if ($it =~ /^(.*::)([^:]+)$/) {
         my ($stash, $name) = ($1, $2);
-        if (!defined %$stash) {
+        if (!%$stash) {
             return;
         }
         if ($re) {
@@ -387,7 +413,9 @@ sub apropos
                 my $stashnm = "$stash$_";
                 /$name/ &&
                     (($stashp && /::$/)
-                     || scalar grep { defined *{$stashnm}{$_} } @types)
+                     || scalar grep {
+                         defined($_ eq 'SCALAR' ? $$stashnm : *{$stashnm}{$_})
+                     } @types)
             } keys %$stash;
         } else {
             defined &$it ? $it : ();
@@ -413,7 +441,7 @@ sub mod_subs
     no strict;
     my $p = shift;
     my $stash = \%{"$p\::"};
-    if (defined $stash) {
+    if (%$stash) {
         grep { defined &{"$p\::$_"} } keys %$stash;
     }
 }
@@ -712,13 +740,62 @@ sub flow
     $_
 }
 
+sub load
+{
+    my $a = shift;
+    no strict;
+    for (@$a) {
+        *{$_->[0]} = $_->[1];
+    }
+}
+
+my %BADVARS;
+undef @BADVARS{qw(%INC @INC %SIG @ISA %ENV @ARGV)};
+
+# magic variables
+sub saveable
+{
+    local $_ = shift;
+    return !/^.[^c-zA-Z]$/ # single-letter stuff (match vars, $_, etc.)
+        && !/^.[\0-\060]/         # magic weirdness.
+        && !/^._</        # debugger info
+        && !exists $BADVARS{$_}; # others.
+}
+
+sub save
+{
+    my ($re) = @_;
+    my @save;
+    $re = qr/(?:^|::)$re/;
+    no strict;                  # no kidding...
+    my_walksymtable {
+        return if /::$/
+            || $stash =~ /^(?:::)?(?:warnings|Config|strict|B)\b/;
+        if (/$re/) {
+            my $name = "$stash$_";
+            if (defined ${$name} and saveable '$'.$_) {
+                push @save, [$name, \$$name];
+            }
+            if (defined *{$name}{HASH} and saveable '%'.$_) {
+                push @save, [$name, \%{$name}];
+            }
+            if (defined *{$name}{ARRAY} and saveable '@'.$_) {
+                push @save, [$name, \@{$name}];
+            }
+        }
+    } '::';
+    print STDERR "$_->[0] " for @save;
+    print STDERR "\n";
+    \@save;
+}
+
 =head2 C<define_shortcut $name, $sub [, $doc [, $shortdoc]]>
 
 Define $name as a shortcut for function $sub.
 
 =cut
 
-    sub define_shortcut
+sub define_shortcut
 {
     my ($name, $doc, $short, $fn);
     if (@_ == 2) {
@@ -775,6 +852,12 @@ sub define_shortcuts
             'Define NAME as a shortcut executing BODY';
     define_shortcut undef => \&Sepia::repl_undef,
         'undef NAME', 'Undefine shortcut NAME';
+    define_shortcut test => \&Sepia::repl_test,
+        'test FILE...', 'Run tests interactively.';
+    define_shortcut load => \&Sepia::repl_load,
+        'load [FILE]', 'Load state from FILE.';
+    define_shortcut save => \&Sepia::repl_save,
+        'save [PATTERN [FILE]]', 'Save variables matching PATTERN to FILE.';
 }
 
 sub repl_help
@@ -786,8 +869,10 @@ sub repl_help
         $args =~ s/\s+$//;
         my $full = $RK{$args};
         if ($full) {
-            print "$RK{$full}    ",
-                flow($width - length $RK{$full} - 4, $REPL_DOC{$full}), "\n";
+            my $short = $REPL_SHORT{$full};
+            my $flow = flow($width - length $short - 4, $REPL_DOC{$full});
+            $flow =~ s/(.)\n/"$1\n".(' 'x (4 + length $short))/eg;
+            print "$short    $flow\n";
         } else {
             print "$args: no such command\n";
         }
@@ -883,21 +968,23 @@ sub who
     no strict;
     if ($re_str =~ /^[\$\@\%\&]/) {
         ## sigil given -- match it
-    sort grep /$re/, map {
-        (defined %{$pack.'::'.$_} ? '%'.$_ : (),
-         defined ${$pack.'::'.$_} ? '$'.$_ : (), # ?
-         defined @{$pack.'::'.$_} ? '@'.$_ : (),
-             defined &{$pack.'::'.$_} ? '&'.$_ : (),
-            )
+        sort grep /$re/, map {
+            my $name = $pack.'::'.$_;
+            (defined *{$name}{HASH} ? '%'.$_ : (),
+             defined *{$name}{ARRAY} ? '@'.$_ : (),
+             defined *{$name}{CODE} ? $_ : (),
+             defined ${$name} ? '$'.$_ : (), # ?
+         )
         } grep !/::$/ && !/^(?:_<|[^\w])/ && /$re/, keys %{$pack.'::'};
     } else {
         ## no sigil -- don't match it
         sort map {
-            (defined %{$pack.'::'.$_} ? '%'.$_ : (),
-             defined ${$pack.'::'.$_} ? '$'.$_ : (), # ?
-             defined @{$pack.'::'.$_} ? '@'.$_ : (),
-         defined &{$pack.'::'.$_} ? $_ : (),
-     )
+            my $name = $pack.'::'.$_;
+            (defined *{$name}{HASH} ? '%'.$_ : (),
+             defined *{$name}{ARRAY} ? '@'.$_ : (),
+             defined *{$name}{CODE} ? $_ : (),
+             defined ${$name} ? '$'.$_ : (), # ?
+         )
         } grep !/::$/ && !/^(?:_<|[^\w])/ && /$re/, keys %{$pack.'::'};
     }
 }
@@ -926,10 +1013,11 @@ sub columnate
 sub repl_who
 {
     my ($pkg, $re) = split ' ', shift;
-    if ($pkg =~ /^\/(.*)\/?$/) {
+    no strict;
+    if ($pkg && $pkg =~ /^\/(.*)\/?$/) {
         $pkg = $PACKAGE;
         $re = $1;
-    } elsif (!$re && !defined %{$pkg.'::'}) {
+    } elsif (!$re && !%{$pkg.'::'}) {
         $re = $pkg;
         $pkg = $PACKAGE;
     }
@@ -946,7 +1034,7 @@ sub methods
         : grep {
             defined *{"$pack\::$_"}{CODE}
         } keys %{$pack.'::'};
-    (@own, defined @{$pack.'::ISA'}
+    (@own, defined *{$pack.'::ISA'}{ARRAY}
          ? (map methods($_, $qualified), @{$pack.'::ISA'}) : ());
 }
 
@@ -980,7 +1068,7 @@ sub repl_package
 {
     chomp(my $p = shift);
     no strict;
-    if (defined %{$p.'::'}) {
+    if (%{$p.'::'}) {
         $PACKAGE = $p;
 #         my $ecmd = '(setq sepia-eval-package "'.$p.'")';
 #         print ";;;###".length($ecmd)."\n$ecmd\n";
@@ -1038,6 +1126,43 @@ sub repl_eval
     }
 }
 
+sub repl_test
+{
+    my ($buf) = @_;
+    my @files;
+    if ($buf =~ /\S/) {
+        $buf =~ s/^\s+//;
+        $buf =~ s/\s+$//;
+        if (-f $buf) {
+            push @files, $buf;
+        } elsif (-f "t/$buf") {
+            push @files, $buf;
+        } else {
+            return;
+        }
+    } else {
+        find({ no_chdir => 1,
+               wanted => sub {
+                   push @files, $_ if /\.t$/;
+            }}, Cwd::getcwd() =~ /t\/?$/ ? '.' : './t');
+    }
+}
+
+sub repl_load
+{
+    my ($file) = split ' ', shift;
+    $file ||= "$ENV{HOME}/.sepia-save";
+    load(retrieve $file);
+}
+
+sub repl_save
+{
+    my ($re, $file) = split ' ', shift;
+    $re ||= '.';
+    $file ||= "$ENV{HOME}/.sepia-save";
+    store save($re), $file;
+}
+
 ## Collects warnings for REPL
 my @warn;
 
@@ -1265,18 +1390,39 @@ sub html_module_list
     return unless $inst;
     my $out;
     open OUT, ">", $file || \$out or return;
-    print OUT "<html><body><ul>";
+    print OUT "<html><body>";
     my $pfx = '';
     my %ns;
     for (package_list) {
         push @{$ns{$1}}, $_ if /^([^:]+)/;
     }
+    # Handle core modules.
+    my %fs;
+    undef $fs{$_} for map {
+        s/.*man.\///; s|/|::|g; s/\.\d(?:pm)?$//; $_
+    } grep {
+        /\.\d(?:pm)?$/ && !/man1/ && !/usr\/bin/ # && !/^(?:\/|perl)/
+    } $inst->files('Perl');
+    my @fs = sort keys %fs;
+    print OUT qq{<h2>Core Modules</h2><ul>};
+    for (@fs) {
+        print OUT qq{<li><a href="$base$_">$_</a>};
+    }
+    print OUT '</ul><h2>Installed Modules</h2><ul>';
+
+    # handle the rest
     for (sort keys %ns) {
+        next if $_ eq 'Perl';   # skip Perl core.
         print OUT qq{<li><b>$_</b><ul>} if @{$ns{$_}} > 1;
         for (sort @{$ns{$_}}) {
-            my @fs = map {
+            my %fs;
+            undef $fs{$_} for map {
                 s/.*man.\///; s|/|::|g; s/\.\d(?:pm)?$//; $_
-            } grep /\.\d(?:pm)?$/, sort $inst->files($_);
+            } grep {
+                /\.\d(?:pm)?$/ && !/man1/
+            } $inst->files($_);
+            my @fs = sort keys %fs;
+            next unless @fs > 0;
             if (@fs == 1) {
                 print OUT qq{<li><a href="$base$fs[0]">$fs[0]</a>};
             } else {
@@ -1289,6 +1435,7 @@ sub html_module_list
         }
         print OUT qq{</ul>} if @{$ns{$_}} > 1;
     }
+
     print OUT "</ul></body></html>\n";
     close OUT;
     $file ? 1 : $out;
@@ -1346,7 +1493,7 @@ Bug reports welcome, patches even more welcome.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2005-2008 Sean O'Rourke.  All rights reserved, some
+Copyright (C) 2005-2009 Sean O'Rourke.  All rights reserved, some
 wrongs reversed.  This module is distributed under the same terms as
 Perl itself.
 
