@@ -1,12 +1,9 @@
 ;;; Sepia -- Simple Emacs-Perl InterAction: ugly, yet effective.
-;;  (a.k.a. Septik -- Sean's Emacs-Perl Total Integration Kludge.)
+;;; (a.k.a. Septik -- Sean's Emacs-Perl Total Integration Kludge.)
 
-;; Author: Sean O'Rourke <seano@cpan.org>
-;; Keywords: Perl, languages
-
-;; Copyright (C) 2004-2009 Sean O'Rourke.  All rights reserved, some
-;;   wrongs reversed.  This code is distributed under the same terms
-;;   as Perl itself.
+;; Copyright (C) 2004-2008 Sean O'Rourke.  All rights reserved, some
+;; wrongs reversed.  This code is distributed under the same terms as
+;; Perl itself.
 
 ;;; Commentary:
 
@@ -1169,7 +1166,6 @@ Does not require loading.")
                  (mod-file "Find the file defining a package.")
                  (apropos "Find subnames matching RE.")
                  (lexicals "Find lexicals for a sub.")
-                 (apropos-module "Find installed modules matching RE.")
                  ))
       (apply #'define-xref-function "Sepia" x))
 
@@ -1290,7 +1286,7 @@ prefix arg, replace the region with the result."
 
 \(i.e. evaluate an expression on each line of region).  With
 prefix arg, replace the region with the result."
-  (interactive "MExpression: \nr\nP")
+  (interactive "MExpression:\nr\nP")
   (sepia-perlize-region-internal
    "do { my $ret='';my $region = "
    (concat "; for (split /(?<=\\n)/, $region, -1) { $ret .= do { " expr
@@ -1301,7 +1297,7 @@ prefix arg, replace the region with the result."
   "Evaluate a Perl expression on the region as a whole.
 
 With prefix arg, replace the region with the result."
-  (interactive "MExpression: \nr\nP")
+  (interactive "MExpression:\nr\nP")
   (sepia-perlize-region-internal
    "do { local $_ = " (concat "; do { " expr ";}; $_ }") beg end replace-p))
 
@@ -1329,15 +1325,6 @@ With prefix arg, replace the region with the result."
 	;; (car (xref-file-modules file))
 	(sepia-buffer-package))))
 
-;;;###autoload
-(defun sepia-apropos-module (name)
-  (interactive "MList modules matching regexp: ")
-  (let ((res (xref-apropos-module name)))
-    (if res
-        (with-output-to-temp-buffer "*Modules*"
-          (display-completion-list res))
-      (message "No modules matching %s." name))))
-
 ;;;###autoload
 (defun sepia-eval-defun ()
   "Re-evaluate the current function and rebuild its Xrefs."
@@ -1537,7 +1524,7 @@ calling `cperl-describe-perl-symbol'."
                   (if (and
                        (let ((bol (save-excursion (beginning-of-line)
                                                   (point))))
-                         (looking-back " *\\(?:use\\|require\\|package\\|no\\)\\s +[^ ]*" bol))
+                         (looking-back " *\\(?:use\\|require\\|package\\) +[^ ]+" bol))
                        (sepia-looks-like-module obj))
                       (sepia-core-version obj)
                       ""))))
